@@ -247,28 +247,26 @@ def test_apcustom_required_options():
 
 
 def test_override_parser():
+    """Test overriding argparse_custom.DEFAULT_ARGUMENT_PARSER"""
     import importlib
 
     from cmd2 import (
-        DEFAULT_ARGUMENT_PARSER,
+        argparse_custom,
     )
 
     # The standard parser is Cmd2ArgumentParser
-    assert DEFAULT_ARGUMENT_PARSER == Cmd2ArgumentParser
+    assert argparse_custom.DEFAULT_ARGUMENT_PARSER == Cmd2ArgumentParser
 
     # Set our parser module and force a reload of cmd2 so it loads the module
     argparse.cmd2_parser_module = 'examples.custom_parser'
     importlib.reload(cmd2)
-    from cmd2 import (
-        DEFAULT_ARGUMENT_PARSER,
-    )
 
     # Verify DEFAULT_ARGUMENT_PARSER is now our CustomParser
     from examples.custom_parser import (
         CustomParser,
     )
 
-    assert DEFAULT_ARGUMENT_PARSER == CustomParser
+    assert argparse_custom.DEFAULT_ARGUMENT_PARSER == CustomParser
 
 
 def test_apcustom_metavar_tuple():
@@ -286,3 +284,58 @@ def test_cmd2_attribute_wrapper():
     new_val = 22
     wrapper.set(new_val)
     assert wrapper.get() == new_val
+
+
+def test_completion_items_as_choices(capsys):
+    """
+    Test cmd2's patch to Argparse._check_value() which supports CompletionItems as choices.
+    Choices are compared to CompletionItems.orig_value instead of the CompletionItem instance.
+    """
+    from cmd2.argparse_custom import (
+        CompletionItem,
+    )
+
+    ##############################################################
+    # Test CompletionItems with str values
+    ##############################################################
+    choices = [CompletionItem("1", "Description One"), CompletionItem("2", "Two")]
+    parser = Cmd2ArgumentParser()
+    parser.add_argument("choices_arg", type=str, choices=choices)
+
+    # First test valid choices. Confirm the parsed data matches the correct type of str.
+    args = parser.parse_args(['1'])
+    assert args.choices_arg == '1'
+
+    args = parser.parse_args(['2'])
+    assert args.choices_arg == '2'
+
+    # Next test invalid choice
+    with pytest.raises(SystemExit):
+        args = parser.parse_args(['3'])
+
+    # Confirm error text contains correct value type of str
+    out, err = capsys.readouterr()
+    assert "invalid choice: '3' (choose from '1', '2')" in err
+
+    ##############################################################
+    # Test CompletionItems with int values
+    ##############################################################
+    choices = [CompletionItem(1, "Description One"), CompletionItem(2, "Two")]
+    parser = Cmd2ArgumentParser()
+    # noinspection PyTypeChecker
+    parser.add_argument("choices_arg", type=int, choices=choices)
+
+    # First test valid choices. Confirm the parsed data matches the correct type of int.
+    args = parser.parse_args(['1'])
+    assert args.choices_arg == 1
+
+    args = parser.parse_args(['2'])
+    assert args.choices_arg == 2
+
+    # Next test invalid choice
+    with pytest.raises(SystemExit):
+        args = parser.parse_args(['3'])
+
+    # Confirm error text contains correct value type of int
+    out, err = capsys.readouterr()
+    assert 'invalid choice: 3 (choose from 1, 2)' in err
